/****************************************************
  File:       birth.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       September 4, 1997

  Purpose:

    Function BIRTH performs the "birth step" for
    adding a new QTL to the model.
****************************************************/

#include "revjump.h"

extern float gennor(float, float);
extern float genunf(float, float);
extern int get_new_interval(int, int, double *);
extern double get_new_locus(int);
extern double dist_rec(double, int, double *);
extern void f1_prob(int, int, double, double, double *);
extern float genunf(float, float);

/* LAPACK ROUTINES */

extern int dgeqrf_(long int *, long int *, double *, long int *,
		   double *, double *, long int *, long int *);
extern int dtrtrs_(char *, char *, char *, long int *, long int *,
                   double *, long int *, double *, long int *,  long int *);
extern int dormqr_(char *, char *, long int *, long int *, long int *,
	           double *, long int *, double *, double *,
		   long int *, double *, long int *, long int *);
extern int dorgqr_(long int *, long int *, long int *, double *,
		   long int *, double *, double *, long int *, long int *);

/***************************************
      Function GET_NEW_INTERVAL
    randomly draws a new QTL interval 
***************************************/

int get_new_interval( int nmark, int nqtl, double *lambda ){
  int i, new_interval, number_of_intervals, lambda_interval;
  double uni_ran, work;

  number_of_intervals = nmark - 1 - nqtl;
  uni_ran = genunf(0.0, 1.0);
  work = number_of_intervals * uni_ran;
  new_interval = work;

  if(nqtl > 0){
    for(i=0;i<nqtl;i++){
      lambda_interval = lambda[i];
      if(lambda_interval < new_interval)
        new_interval = new_interval + 1;
    }
  }
  return new_interval;
}

/***************************************
      Function GET_NEW_LOCUS
    randomly draws a QTL position
    within the new interval.
***************************************/

double get_new_locus(int new_interval){
  double low, high, uni_ran;

  low = new_interval;
  high = new_interval + 1;
  uni_ran = genunf(low, high);

  return uni_ran;
}

/*************************************
     Function GET_NEW_QTL_GENOTYPE

   computes genotypes of the new qtl 
*************************************/

int *get_new_qtl_genotype( int ntrait, int marker_genotype[][MAXMARKER], 
                           double *dist, double *recombination,
                           double new_position, int *new_qtl_genotype,
			   double *prob){

  /* int *new_qtl_genotype = (int *) malloc( 100 * sizeof(int) ); */
  int i,j, marker_locus, marker1, marker2;
  double r1, R, uni_ran;
  /* double *prob = (double *) malloc(2*sizeof(double)); */

  marker_locus = new_position;
  r1 = dist_rec( new_position, marker_locus, &dist[0] );
  R = recombination[marker_locus];

  for(i=0;i<ntrait;i++){
    marker1 = marker_genotype[i][marker_locus];
    marker2 = marker_genotype[i][marker_locus+1];
    f1_prob( marker1, marker2, r1, R, &prob[0] );
    uni_ran = genunf(0.0, 1.0);
    new_qtl_genotype[i] = random_marker(uni_ran, prob[0]);
  }
  return new_qtl_genotype;
}

/*************************************
    Function GET_NEW_EFFECT
 get effect of the new putative locus
*************************************/

void get_birth_effect(long int ntrait, long int nqtl, 
		      int qtl_genotype[][MAXQTL],
                      int *new_qtl_genotype, double *y,
		      double *modified_parms, 
		      double *new_effect_parms, double sigmasq,
		      double *work_qtl, double *test_o, 
		      double *temp_qtl_geno, double *work_trait, 
		      double *tau, double *test_tau, double *work,
		      double *test_effect, double *up_tri){
  char   side, trans, uplo, diag;
  long int    i,j, info, ncol_q; 
  long int    lwork = nqtl+1;
  long int nrhs;
  double scalar_v, opq_sq, opq_prime_opy, qpy, qpq;
  double mu, new_effect;
/*
  double *work_qtl = (double *) malloc( (MAXTRAIT*lwork)*sizeof(double ) );
  double *test_o = (double *) malloc( (MAXTRAIT*lwork)*sizeof(double ) );
  double *temp_qtl_geno = (double *) malloc(MAXTRAIT*sizeof(double));
  double *work_trait = (double *) malloc(MAXTRAIT*sizeof(double));
  double *tau = (double *) malloc(MAXQTL*sizeof(double));
  double *test_tau = (double *) malloc(MAXQTL*sizeof(double));
  double *work = (double *) malloc(MAXQTL*sizeof(double));
  double *test_effect = (double *) malloc(MAXQTL*sizeof(double));
  double *up_tri = (double *) malloc( (MAXQTL*MAXQTL)*sizeof(double));
*/

  mu = modified_parms[0];
  for(j=0;j<nqtl;j++)
    test_effect[j] = modified_parms[j+1];

  for(i=0;i<ntrait;i++){
    work_qtl[i] = 1;
    temp_qtl_geno[i] = new_qtl_genotype[i];
    work_trait[i] = y[i];
  }
  for(j=0;j<nqtl;j++){
    for(i=0;i<ntrait;i++){
      work_qtl[ntrait+i+j*ntrait] = qtl_genotype[i][j];
    }
  }

/* Get OU decomposition of work_qtl, where the columns of
   O are orthogonal and U is upper-triangular */

  info = dgeqrf_(&ntrait, &lwork, &work_qtl[0], &ntrait,
                 &tau[0], &work[0], &lwork, &info);

/* Get the orthonormal matrix O. First create a test O matrix
   called test_o so that we don't loose work_qtl in this
   process */

for(j=0; j<=nqtl; j++){
  test_tau[j] = tau[j];
  for(i=0; i<ntrait; i++){
    test_o[i+j*ntrait] = work_qtl[i+j*ntrait];
  }
}

  info = dorgqr_(&ntrait, &lwork, &lwork, &test_o[0], &ntrait,
		 &test_tau[0], &work[0], &lwork, &info);

/*
for(j=0; j<=nqtl; j++){
  for(i=0; i<ntrait; i++){
    printf("test o %d %d is %f\n", i, j, test_o[i+j*ntrait]);
  }
}
*/

/* Get O'Q, where Q is new_qtl_genotype */

  side = 'L';
  trans = 'T';   /* side and trans say that we need O'Q */
  ncol_q = 1;    /* says that Q is a column vector */

  info = dormqr_(&side, &trans, &ntrait, &ncol_q, &lwork,
                 &work_qtl[0], &ntrait, &tau[0], &temp_qtl_geno[0],
                 &ntrait, &work[0], &lwork, &info);


/* Get O'Y, Y=work_trait */

  info = dormqr_(&side, &trans, &ntrait, &ncol_q, &lwork,
                 &work_qtl[0], &ntrait, &tau[0], &work_trait[0],
                 &ntrait, &work[0], &lwork, &info);

  qpq = 0;
  opq_sq = 0;
  opq_prime_opy = 0;
  qpy = 0;
  scalar_v = 0;

  for(i=0;i<ntrait;i++){
    qpq = qpq + new_qtl_genotype[i] * new_qtl_genotype[i];
    qpy = qpy + new_qtl_genotype[i] * y[i];
  }

  for(j=0;j<lwork;j++){ 
    opq_sq = opq_sq + temp_qtl_geno[j] * temp_qtl_geno[j];
    /*opq_prime_opy = opq_prime_opy + work_trait[j] * work_trait[j];*/
    opq_prime_opy = opq_prime_opy + temp_qtl_geno[j] * work_trait[j];
  }

  scalar_v = qpq - opq_sq;

  new_effect = (double) gennor(0.0, 1.0);
  new_effect_parms[0] = new_effect;
  new_effect_parms[1] = scalar_v;

/*
printf("\n");
printf("normal random number is %f\n", new_effect);
printf("scalar v is %f\n", scalar_v);
printf("sigmasq is %f\n", sigmasq);
printf("\n");
*/

  if(scalar_v > 0.0000001){
    new_effect = (qpy - opq_prime_opy) / scalar_v +
                  sqrt(sigmasq) * new_effect / sqrt(scalar_v);

/*
    printf("qpy is %f\n", qpy);
    printf("opq prime opy is %f\n", opq_prime_opy);
    printf("term 1 is %f\n", (qpy - opq_prime_opy) / scalar_v);
    printf("term 2 is %f\n",  sqrt(sigmasq) * new_effect / sqrt(scalar_v));
*/
  }
/* printf("new effect is %f\n", new_effect); */

  /* CREATE THE UPPER TRIANGULAR MATRIX UP_TRI */
  for(i=0;i<lwork;i++)
    for(j=0;j<=i;j++){
      up_tri[j+lwork*i] = work_qtl[j+ntrait*i];
    }


  /* NEED TO COMPUTE (X'X)^{-1}X'Q TYPE OPERATION. 
     CAN USE DTRTRS FUNCTION FROM LAPACK. NEED TO
     DEFINE UPLO, TRANS AND DIAG TO TELL DTRTRS
     WHAT IT MUST DO. THESE ARE DEFINED FIRST AND
     DTRTRS FUNCTION IS USED THEN */

  uplo = 'U';
  trans = 'N';
  diag = 'N';
  nrhs = 1;

  info = dtrtrs_(&uplo, &trans, &diag, &lwork, &nrhs, &up_tri[0],
                 &lwork, &temp_qtl_geno[0], &lwork, &info);

/*
printf("\n");
printf("temp qtl geno\n");
for(j=0; j<=nqtl; j++)
  printf("temp qtl geno %d is %f\n", j, temp_qtl_geno[j]);
printf("\n");
*/

for(i=0;i<lwork;i++){
  if(temp_qtl_geno[i] < -999) temp_qtl_geno[i] = 0.0;
}

/* printf("before modification\n");
printf("mu is %f\n", mu);
for(j=0; j<nqtl; j++)
  printf("effect %d is %f\n", j, test_effect[j]);
printf("end before modification\n");

printf("\n");
printf("new effect is %f\n", new_effect);
printf("\n");
*/

  mu = mu - temp_qtl_geno[0] * new_effect;
  for(j=0;j<nqtl;j++){
    test_effect[j] = test_effect[j] - temp_qtl_geno[j+1] * new_effect;
  }

  modified_parms[0] = mu;
  for(j=0;j<=nqtl;j++)
    modified_parms[j+1] = test_effect[j];

  modified_parms[nqtl+1] = new_effect;

/*
 free (work_qtl);
 free (test_o);
 free (temp_qtl_geno);
 free (work_trait);
 free (tau);
 free (test_tau);
 free (work);
 free (up_tri);
 free (test_effect);
*/

}
