/**************************************************************
  File:         initgeno.c
  Written by:   Jaya Satagopan (satago@biost.mskcc.org)
  Date:         August 27, 1997
  Updated:      September 2, 1997

  Purpose:
  -------
    These functions find the index of missing markers and traits.

**************************************************************/

#include "revjump.h"

extern double haldane(double);
extern double dist_rec(double, int, double *);
extern double halrec(double, double);
extern float genunf(float, float);
extern float gennor(float, float);

/********************************************
    FUNCTION FINDMISS_MARKER
     finds missing markers
********************************************/

void findmiss_marker( int ntrait, int nmark, int marker[][MAXMARKER],
                      int miss_mark[][MAXMARKER] ){
  int i,j,k;
  for(i=0; i<nmark; i++){
    k=0;
    for(j=0; j<ntrait; j++){
      if( ismiss(marker[j][i]) == 1 ){
        k = k+1;
        miss_mark[k][i] = j;
      }
    }
    miss_mark[0][i] = max( 0, k );
  }
}

/********************************************
    FUNCTION FINDMISS_TRAIT
     finds missing traits
********************************************/

void findmiss_trait( int ntrait, double *y, double *ymiss){
  int j, k;

  k= 0;
  for(j=0; j<ntrait; j++){
    if( ismiss(y[j]) == 1 ){
      k = k+1;
      ymiss[k] = j;
    }
  }
  ymiss[0] = max(1, k);
}

/********************************************
    FUNCTION INITMISS_MARKER
    initializes missing markers
********************************************/

void initmiss_marker( int ntrait, int nmark, double *recombination, 
              int miss_mark[][MAXMARKER], int marker[][MAXMARKER] ){
  int i, j, k, mark, mark1, mark2;
  double r1, R, prob[2], uni_ran;
  for( j=0; j<ntrait; j++ ){
    i = 0;
    while( (i<(nmark-1)) && (ismiss(marker[j][i]) == 1) )
      i++;
    if( i>0 ){
      /* Backtrack to fill missing marker data */
      if( (i==(nmark-1)) && (ismiss(marker[j][nmark-1]==1)) ){
  uni_ran = genunf(0.0,1.0);        /* all markers missing */
     marker[j][nmark-1] = random_marker(uni_ran, 0.5);
      }
      R = 0.5;        /* actual backtracking happens below */
      for(k=(i-1); k>=0; k--){
     r1 = recombination[k];       /***CHECK THIS**/
  mark = marker[j][k+1];
  f1_prob(mark, mark, r1, R, &prob[0]);
   uni_ran = genunf(0.0,1.0);
      marker[j][k] = random_marker(uni_ran, prob[0]);
      }
    }
    if( i<(nmark-1) ){
      for(k=(i+1);k<(nmark-1);k++){   /**CHECK THIS**/
 if( ismiss(marker[j][k])==1 ){
    r1 = recombination[k-1];
        if( ismiss(marker[j][k+1])==1 )
           R = 0.5;
          else
          R = halrec(r1, recombination[k]);
     mark1 = marker[j][k-1];
         mark2 = marker[j][k+1];
         f1_prob(mark1, mark2, r1, R, &prob[0]);
         uni_ran = genunf(0.0, 1.0);
     marker[j][k] = random_marker(uni_ran, prob[0]);
       }
      }
      if( ismiss(marker[j][nmark-1])==1 ){
      r1 = recombination[nmark-2];
    R = 0.5;
        mark = marker[j][nmark-2];
      f1_prob(mark, mark, r1, R, &prob[0]);
   uni_ran = genunf(0.0, 1.0);
     marker[j][nmark-1] = random_marker(uni_ran, prob[0]);
      }
    }
  }
}


/********************************************
         Function INITQTL_GENO

  initializes qtl genotypes given their 
  location and flanking marker genotypes
********************************************/

void initqtl_geno( int ntrait, int nmark, int nqtl, double *lambda,
                   double *dist, double *recombination,
            int marker_genotype[][MAXMARKER], 
              int qtl_genotype[][MAXQTL] ){

  int iqtl, jobs, flank_mark_dist;
  double dval, r1, R, temp_lambda, prob[2], uni_ran;

  uni_ran = genunf(0,1);

  for(iqtl=0;iqtl<nqtl;iqtl++){
    flank_mark_dist = lambda[iqtl];
    temp_lambda = lambda[iqtl];
    dval = dist[flank_mark_dist] * (temp_lambda - flank_mark_dist);
    r1 = dist_rec( temp_lambda, flank_mark_dist, &dist[0] );
    R = recombination[flank_mark_dist];

    for(jobs=0; jobs<ntrait; jobs++){
      f1_prob( marker_genotype[jobs][flank_mark_dist],
               marker_genotype[jobs][flank_mark_dist+1], r1, R, &prob[0] );
      uni_ran = genunf(0.0, 1.0);
      qtl_genotype[jobs][iqtl] = random_marker(uni_ran, prob[0]);
    }
  }
}

/**************************************************
         Function DIST_REC_i      

converts distance between markers and qtl 
recombination using Haldane mapping function.
**************************************************/

double dist_rec( double lambda_val, int marker_dist, double *dist){

  double dval, r1;

  dval = dist[marker_dist] * (lambda_val - marker_dist);
  r1 = haldane(dval);
  return r1;
}

/*********************************
        Function SETMISSY

set missing y values based on the 
model

Date:  September 3, 1997
*********************************/

void setmissy( int ntrait, int nqtl, double *ymiss, 
               double mu, double *effect, double sigmasq, 
         int qtl_genotype[][MAXQTL], double *y ){
  int jmiss, kqtl, work;
  double meany,sdy;
  int i,j;

  for(jmiss=1; jmiss<=ymiss[0]; jmiss++){
    work = ymiss[jmiss];
    meany = 0.0;
    for(kqtl=0;kqtl<nqtl;kqtl++)
      meany = meany + qtl_genotype[work][kqtl] * effect[kqtl];
    meany = mu + meany;
    sdy = sqrt(sigmasq);

    y[work] = gennor( meany, sdy );
  }
}

/********************************
  Function SET_MISS_MARKER

  Date: September 8, 1997
********************************/

void set_miss_marker(int ntrait, int nmark, int imark,
                     double *recombination, int miss_mark[][MAXMARKER],
                     int marker_genotype[][MAXMARKER]){
  int jobs, jmiss, imark1, work1, work2;
  double r1, R, uni_ran, prob_val[2];
  /* double *prob_val = (double *) malloc(2*sizeof(double)); */

  if( (imark>0) && (imark < (nmark-1)) ){
    r1 = recombination[imark-1];
    R = halrec(r1, recombination[imark]);
    for(jmiss=1; jmiss<=miss_mark[0][imark]; jmiss++){
      jobs = miss_mark[jmiss][imark];
      work1 = marker_genotype[jobs][imark-1];
      work2 = marker_genotype[jobs][imark+1];
      f1_prob(work1, work2, r1, R, &prob_val[0]);
      uni_ran = genunf(0.0, 1.0);
      marker_genotype[jobs][imark] = random_marker(uni_ran, prob_val[0]);
    }
  }
  else{
    R = 0.5;
    if(imark==0){
      r1 = recombination[imark];
      imark1 = imark+1;
    }
    else{
      r1 = recombination[imark-1];
      imark1 = imark - 1;
    }
    for(jmiss=1; jmiss <= miss_mark[0][imark]; jmiss++){
      jobs = miss_mark[jmiss][imark];
      work1 = marker_genotype[jobs][imark1];
      f1_prob(work1, work1, r1, R, &prob_val[0]);
      uni_ran = genunf(0.0, 1.0);
      marker_genotype[jobs][imark] = random_marker(uni_ran, prob_val[0]);
   }
  }
}
