/**************************************************************
  File:         read_data.c
  Written by:   Jaya Satagopan (satago@biost.mskcc.org)
  Date:         August 26, 1997

  Purpose:
  -------
    This file has 3 functions. 

    The first function is GETDATA of type VOID.This reads 
    trait, marker, distance and initial values from different 
    files. 

    The second function is GETCUMDIST_RECOM if type VOID. This 
    function obtains cumulative distance between markers and 
    recombination fraction between pairs of markers, given 
    distance between them.

    The third function is HALDANE of type DOUBLE. This function
    converts distance into recombination fraction based on the
    haldane assumption.

**************************************************************/

#include "revjump.h"

extern double haldane( double );

/*************************************************
             FUNCTION GETDATA
*************************************************/

void getdata( int *nvals, double *y, int marker[][MAXMARKER], double *dist,
	      double *model_parms, double *lambda, double *hyper_parms ){

  /* open files to read and write */
  int end_of_file1, end_of_file2;
  FILE *read_n;
  FILE *read_trait;
  FILE *read_marker;
  FILE *read_distance;
  FILE *write_states;
  FILE *write_rejection;

  int revjump, nt, nm, niter, nby, nqtl;
  double mu, sigmasq, tempeffect, temppos;
  double trait, tempdist;
  int tempmark;
  int i,j;

  double mu_prior_mn, mu_prior_var, sigsq_prior_a1, sigsq_prior_a2,
	 effect_prior_mn, effect_prior_var, qtl_prior_mn;

  /***************************************** 
               READ NTRAIT-NMARK DATA
  *****************************************/

  read_n = fopen("nval.dat", "r");
  end_of_file1 = fscanf( read_n, " %d %d %d %d %d %d %lf %lf", 
		  &revjump, &nt, &nm, &niter, &nby, &nqtl, &mu, &sigmasq ); 

/*************************************************************************
   the parameter revjump is binary. revjump=1 means do 
   reversible jump mcmc computations. if revjump=0, no reversible
   jump mcmc, do only regular mcmc as in satagopan et al (1996).
*************************************************************************/

  model_parms[0] = mu;
  model_parms[1] = sigmasq;

  if( nqtl > 0 ){
    for( i=0; i<nqtl; i++ ){
      end_of_file1 = fscanf( read_n, "%lf", &tempeffect );
      model_parms[(i+2)] = tempeffect;
    }
    for( i=0; i<nqtl; i++ ){
      end_of_file1 = fscanf( read_n, "%lf", &temppos );
      lambda[i] = temppos;
    }
  }

  end_of_file1 = fscanf( read_n, "%lf %lf %lf %lf", 
			 &mu_prior_mn, &mu_prior_var,
			 &sigsq_prior_a1, &sigsq_prior_a2 );

  hyper_parms[0] = mu_prior_mn;
  hyper_parms[1] = mu_prior_var;
  hyper_parms[2] = sigsq_prior_a1;
  hyper_parms[3] = sigsq_prior_a2;

  end_of_file1 = fscanf( read_n, "%lf %lf", 
			 &effect_prior_mn, &effect_prior_var );
  hyper_parms[4] = effect_prior_mn;
  hyper_parms[5] = effect_prior_var;

/* qtl prior mean is required only if doing reversible jump mcmc
   computations as the following "if" statement indicates */

  if( revjump == 1) {
    end_of_file1 = fscanf( read_n, "%lf", &qtl_prior_mn );
    hyper_parms[6] = qtl_prior_mn;
  }
  else{
    hyper_parms[6] = nqtl;
  }

  fclose(read_n);

  nvals[0] = revjump;
  nvals[1] = nt;
  nvals[2] = nm;
  nvals[3] = niter;
  nvals[4] = nby;
  nvals[5] = nqtl;

  /***************************************** 
               READ TRAIT DATA
  *****************************************/

  read_trait = fopen("trait.y", "r");
  for( i=0; i< nt; i++ ){
    end_of_file1 = fscanf( read_trait, "%lf", &trait );
    if( end_of_file1 != EOF ){
      y[i] = trait;
    }
  }
  fclose(read_trait);

  /***************************************** 
               READ MARKER DATA
  *****************************************/

  read_marker = fopen("marker.mark", "r");
  for( i=0; i< nm; i++ ){
    for( j=0; j< nt; j++ ){
      end_of_file2 = fscanf(read_marker, "%d", &tempmark);
      if( end_of_file2 != EOF ){
        marker[j][i] = tempmark;
      }
    }
  }
  fclose(read_marker);

  /***************************************** 
      READ INTER-MARKER DISTANCE DATA 
  *****************************************/

  read_distance = fopen("marker.dist", "r");
  for( i=0; i<(nm-1); i++ ){
    end_of_file1 = fscanf(read_distance, "%lf", &tempdist);
    if( end_of_file1 != EOF )
      dist[i] = tempdist;
  }
}

/*************************************************
             FUNCTION GETCUMDIST_RECOM 
*************************************************/

void getcumdist_recom( int nmark, double *dist, double *cumdist, 
		       double *recombination ){
  int i;
  cumdist[0] = 0.0;
  cumdist[1] = dist[0];
  dist[0] = dist[0] / 100.0;
  recombination[0] = haldane( dist[0] );
  for( i=1; i<(nmark-1); i++ ){
    cumdist[i+1] = cumdist[ i ] + dist[i];
    dist[i] = dist[i] / 100.0;
    recombination[i] = haldane( dist[i] );
  }
}

/*************************************************
             FUNCTION HALDANE
*************************************************/

double haldane( double dist_val ){
  double recval;
  recval = 0.5 * ( 1 - exp( -2.0 * dist_val ) );
  return recval;
}
